#include "NotificationArea.hpp"
#include "Globals.hpp"
#include <QtGui/QGuiApplication>
#include <QtGui/QScreen>
#include <QtGui/QPainter>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QBoxLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QAction>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QScrollBar>
#include <QtWidgets/QLabel>

//=============================================================================

NotificationArea::NotificationArea( QWidget *parent )
    : QFrame( parent, Qt::FramelessWindowHint | Qt::Tool | Qt::Dialog | Qt::WindowStaysOnTopHint )
    , m_actClose( NULL )
    , m_CenterLayout( NULL )
    , m_ScreenWidget( NULL )
{
    m_actClose = new QAction( tr( "Close" ), this );
    connect( m_actClose, SIGNAL( triggered() ), this, SLOT( close() ) );

    QLabel *headerLabel = new QLabel( tr( "Notifications" ), this );
    QFont headerFont = headerLabel->font();
    headerFont.setPointSizeF( headerFont.pointSizeF() * 1.8 );
    headerLabel->setFont( headerFont );
    QToolButton *closeButton = new QToolButton( this );
    closeButton->setDefaultAction( m_actClose );
    closeButton->setAutoRaise( true );
    closeButton->setFocusPolicy( Qt::NoFocus );
    closeButton->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );

    QHBoxLayout* headerLayout = new QHBoxLayout();
    headerLayout->addWidget( headerLabel );
    headerLayout->addWidget( closeButton );
    headerLayout->setMargin( 6 );
    QWidget *topWidget = new QWidget( this );
    topWidget->setLayout( headerLayout );

    QWidget *centerWidget = new QWidget();
    centerWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    m_CenterLayout = new QVBoxLayout();
    centerWidget->setLayout( m_CenterLayout );

    QToolButton *settingsButton = new QToolButton( this );
    settingsButton->setDefaultAction( new QAction( tr( "Settings" ), this ) );
    settingsButton->setAutoRaise( true );
    settingsButton->setFocusPolicy( Qt::NoFocus );
    QHBoxLayout *buttonLayout = new QHBoxLayout();
    buttonLayout->addWidget( settingsButton );
    buttonLayout->addSpacerItem( new QSpacerItem( 5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum ) );

    QVBoxLayout* bottomLayout = new QVBoxLayout;
    bottomLayout->addWidget( centerWidget );
    bottomLayout->addLayout( buttonLayout );
    bottomLayout->setMargin( 6 );
    QWidget *bottomWidget = new QWidget( this );
    bottomWidget->setLayout( bottomLayout );

    QVBoxLayout* layout = new QVBoxLayout();
    layout->addWidget( topWidget );
    layout->addWidget( bottomWidget );
    layout->setMargin( 0 );
    layout->setSpacing( 0 );
    setLayout( layout );
}

NotificationArea::~NotificationArea()
{

}

void NotificationArea::showEvent( QShowEvent* e )
{
    QFrame::showEvent( e );
    updatePositioning();
}

void NotificationArea::paintEvent( QPaintEvent* e )
{
    QFrame::paintEvent( e );
    QPainter p( this );
    p.setPen( Qt::gray );
    p.drawLine( rect().topLeft(), rect().bottomLeft() );
}

void NotificationArea::updatePositioning()
{
    //update position and size of notification panel
    //, it should fill the right side of the screen
    //occupying 1/5 of the screen

    //get screen data
    QGuiApplication::sync();
    QRect screenRect;
    QScreen *screen = QGuiApplication::primaryScreen();
    screenRect = screen->availableGeometry();

    //debug output of screen
    displayScreenData( screen );

    //compute geometry of notification rectangle
    int notificationSize = qMin( 400, screenRect.width() / 5 );
    QRect notificationRect( screenRect.topRight() + QPoint( -notificationSize, 0 ), screenRect.bottomRight() );

    //appy computed geometry to notification widget
    resize( notificationRect.size() );
    move( screenRect.topRight() + QPoint( -width() + DPI_X_Scaled( 1, this ), 0 ) );
}

void NotificationArea::displayScreenData( const QScreen *screen )
{
    //displaying current state of screen on m_ScreenWidget

    delete m_ScreenWidget;
    m_ScreenWidget = new QWidget();
    m_CenterLayout->addWidget( m_ScreenWidget );
    
    QWidget *screenWidget = new QWidget();
    QGridLayout *layout = new QGridLayout();
    int row = 0;
    screenWidget->setLayout( layout );
    screenWidget->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );

    layout->addWidget( new QLabel( tr( "<b>Display data</b>" ) ), row++, 0, 1, 2 );
    addItem( layout, &row, tr( "size" ), new GeoWidget( screen->size() ) );
    addItem( layout, &row, tr( "geometry" ), new GeoWidget( screen->geometry() ) );
    addItem( layout, &row, tr( "physicalSize" ), new GeoWidget( screen->physicalSize() ) );
    addItem( layout, &row, tr( "physicalDotsPerInchX" ), new QLabel( QString( "%1" ).arg( screen->physicalDotsPerInchX() ) ) );
    addItem( layout, &row, tr( "physicalDotsPerInchY" ), new QLabel( QString( "%1" ).arg( screen->physicalDotsPerInchY() ) ) );
    addItem( layout, &row, tr( "physicalDotsPerInch" ), new QLabel( QString( "%1" ).arg( screen->physicalDotsPerInch() ) ) );
    addItem( layout, &row, tr( "logicalDotsPerInchX" ), new QLabel( QString( "%1" ).arg( screen->logicalDotsPerInchX() ) ) );
    addItem( layout, &row, tr( "logicalDotsPerInchY" ), new QLabel( QString( "%1" ).arg( screen->logicalDotsPerInchY() ) ) );
    addItem( layout, &row, tr( "logicalDotsPerInch" ), new QLabel( QString( "%1" ).arg( screen->logicalDotsPerInch() ) ) );
    addItem( layout, &row, tr( "devicePixelRatio" ), new QLabel( QString( "%1" ).arg( screen->devicePixelRatio() ) ) );

    QVBoxLayout *screenLayout = new QVBoxLayout();
    m_ScreenWidget->setLayout( screenLayout );
    screenLayout->addWidget( screenWidget );
}

void NotificationArea::addItem( QGridLayout *layout, int *row, const QString &description, QWidget *widget )
{
    layout->addWidget( new QLabel( description ), *row, 0 );
    layout->addWidget( widget, (*row)++, 1 );
}

//=============================================================================

GeoWidget::GeoWidget( const QSize &size, QWidget *parent )
    : QWidget( parent )
{
    QLabel *txt = new QLabel( tr( "%1 x %2" ).arg( size.width() ).arg( size.height() ) );
    QHBoxLayout *layout = new QHBoxLayout();
    layout->setContentsMargins( 0, 0, 0, 0 );
    layout->setMargin( 0 );
    layout->addWidget( txt );
    setLayout( layout );
}

GeoWidget::GeoWidget( const QSizeF &size, QWidget *parent )
    : QWidget( parent )
{
    QLabel *txt = new QLabel( tr( "%1 x %2" ).arg( size.width() ).arg( size.height() ) );
    QHBoxLayout *layout = new QHBoxLayout();
    layout->setContentsMargins( 0, 0, 0, 0 );
    layout->setMargin( 0 );
    layout->addWidget( txt );
    setLayout( layout );
}

GeoWidget::GeoWidget( const QRect &rect, QWidget *parent )
    : QWidget( parent )
{
    QLabel *txt = new QLabel( tr( "(%1, %2) - %3 x %4" )
        .arg( rect.x() )
        .arg( rect.y() )
        .arg( rect.width() )
        .arg( rect.height() ) );
    QHBoxLayout *layout = new QHBoxLayout();
    layout->setContentsMargins( 0, 0, 0, 0 );
    layout->setMargin( 0 );
    layout->addWidget( txt );
    setLayout( layout );
}

GeoWidget::~GeoWidget()
{

}

//=============================================================================